from flask_admin.menu import MenuLink
from wtforms.fields import PasswordField
from flask_admin.form import fields
from flask_admin.model.template import EndpointLinkRowAction
from flask_admin.contrib.sqla import ModelView
from flask_admin import AdminIndexView, expose
from flask_security import login_required, current_user, utils

from skidward.models import Task
from skidward.backend import RedisBackend


class UserAdmin(ModelView):
    form_excluded_columns = ("password",)

    def is_accessible(self):
        return current_user.has_role("admin")

    def scaffold_form(self):
        form_class = super(UserAdmin, self).scaffold_form()
        form_class.password2 = PasswordField("New Password")
        return form_class

    def on_model_change(self, form, model, is_created):
        if len(model.password2):
            model.password = utils.hash_password(model.password2)


class RoleAdmin(ModelView):
    def is_accessible(self):
        return current_user.has_role("admin")


class TaskView(ModelView):

    column_extra_row_actions = [
        EndpointLinkRowAction(
            icon_class="glyphicon glyphicon-play",
            endpoint="run_task.run_task",
            title="Run_Task",
            id_arg="task_id",
        )
    ]

    form_overrides = {"context": fields.JSONField}


class JobView(ModelView):

    column_list = ["task", "ran_at", "state"]
    column_labels = dict(task="Jobs", state="Status")


class SkidwardView(AdminIndexView):
    @expose("/")
    @login_required
    def index(self):
        return self.render("index.html")


class RunView(AdminIndexView):
    @expose("/run/<task_id>")
    @login_required
    def run_task(self, task_id):
        task = Task.query.get(task_id)
        redis_client = RedisBackend()
        redis_client.lpush("MANUAL_RUN", task_id)
        return self.render("task/run_task.html", name=task.name, context=task.context)


class LoginMenuLink(MenuLink):
    def is_accessible(self):
        return not current_user.is_authenticated


class LogoutMenuLink(MenuLink):
    def is_accessible(self):
        return current_user.is_authenticated



from typing import Optional, Dict
import pkg_resources as pkg

import pytest

from worker_detector.namespace_module_manager import NamespaceModuleManager


@pytest.fixture
def real_namespace() -> str:
    return "skidward.real.name.space"


class MockEntryPoint(pkg.EntryPoint):
    def __init__(self, name, module_name):
        self.name = name
        self.module_name = module_name

    def load(self, *args, **kwargs) -> pkg.EntryPoint:
        return MockEntryPoint(self.name, self.module_name)


class MockNamespaceModuleManager(NamespaceModuleManager):
    def _get_module_entry_points_on_namespace(self) -> Dict[str, pkg.EntryPoint]:
        if self.namespace == "skidward.real.name.space":
            return {
                "real_name": MockEntryPoint("real-name", "real_name"),
                "other_name": MockEntryPoint("other-name", "other_name"),
            }

        return super()._get_module_entry_points_on_namespace()


@pytest.fixture
def mock_namespace_module_manager() -> MockNamespaceModuleManager:
    return MockNamespaceModuleManager("skidward.real.name.space")


def mock_create_namespace_module_manager(namespace: str) -> MockNamespaceModuleManager:
    return MockNamespaceModuleManager(namespace)



import os
import time

from skidward import JobStatus
# import before skidward.models to avoid circular imports
from skidward import web
from skidward.backend import RedisBackend
from skidward.models import db, Job


WAKEUP_TIMER = int(os.getenv("WAKEUP_TIMER"))


def job_exists(job, redis_client):
    jobs = redis_client.lrange("jobs", 0, -1)
    return str(job).encode("utf-8") in jobs


def add_jobs_to_redis(redis_client, jobs, session):

    for job in jobs:
        if not job_exists(job, redis_client):
            redis_client.lpush("jobs", job)
            db_job = session.query(Job).filter(Job.id == job)
            db_job.update({"state": JobStatus.RUNNING})
            session.commit()


def get_jobs(redis_backend):
    jobs = db.session.query(Job).filter(Job.state == JobStatus.READY)

    jobs_id = [job.id for job in jobs]

    if jobs_id:
        add_jobs_to_redis(redis_backend, jobs_id, db.session)
    else:
        print("No job ready to run")


def main():

    redis_backend = RedisBackend()
    while True:
        get_jobs(redis_backend)
        time.sleep(WAKEUP_TIMER)


if __name__ == "__main__":
    main()



from unittest import mock
import pytest

from skidward import JobStatus

# import before skidward.models to avoid circular imports
from skidward import web
from skidward.backend import RedisBackend
from skidward.models import Job


@pytest.fixture(autouse=True)
def enable_testing(monkeypatch):
    monkeypatch.setenv("TESTING", "True")


class Jobs(Job):
    def __init__(self, id, state):
        self.id = id
        self.state = state


class JobGenerator:
    def __init__(self):
        self.jobs = []

    def __iter__(self):
        for job in self.jobs:
            yield job

    def update(self, *args, **kwargs):
        pass

    def append(self, job):
        self.jobs.append(job)


class SetupDummyBackend:
    def __init__(self):
        self.dummy_backend = RedisBackend()
        self.dummy_backend.lpush("jobs", 1)

        self.mock_jobs = JobGenerator()
        self.mock_jobs.append(Jobs(id=1, state=JobStatus.READY))


@pytest.fixture()
def setup_backend():
    get_backend = SetupDummyBackend()
    yield get_backend
    get_backend.dummy_backend.erase()


@pytest.fixture()
def backend(setup_backend):
    dummy_backend = setup_backend.dummy_backend
    yield dummy_backend


@pytest.fixture()
def session(setup_backend):
    mock_db = mock.Mock()
    mock_jobs = setup_backend.mock_jobs
    mock_db.query().filter.return_value = mock_jobs

    return mock_db


@pytest.fixture()
def add_job_to_db(setup_backend):
    def append_job_table(id, state):
        setup_backend.mock_jobs.append(Jobs(id, state))

    return append_job_table



import pytest
import logging

import redis
from redis_log_handler.RedisLogHandler import RedisLogHandler


@pytest.fixture
def redis_client():
    return redis.Redis(host='localhost', port=6379, password=None)


class LogTestHelper(object):
    """Used to distribute helper methods through a fixture."""
    @staticmethod
    def generate_logger(name: str, level: str, channel: str) -> logging.Logger:
        logger = logging.getLogger(name)
        logger.addHandler(RedisLogHandler(channel))
        logger.setLevel(level)

        return logger

    @staticmethod
    def generate_subscriber_on_channel(redis_client: redis.Redis, channel: str) -> redis.client.PubSub:
        r = redis_client.pubsub()
        r.subscribe(channel)
        return r

    @staticmethod
    def close_connection_to_channel(*args: redis.client.PubSub):
        for connection in args:
            connection.close()

    @staticmethod
    def is_message_data_empty(message: dict) -> bool:
        is_empty = True
        if message is None:
            return is_empty

        is_empty = message.get('data', False) == 1
        return is_empty


@pytest.fixture
def log_test_helper() -> LogTestHelper:
    return LogTestHelper()



from typing import List, Optional

from pkg_resources import EntryPoint

from worker_detector.namespace_module_manager import NamespaceModuleManager


__NAMESPACE = "skidward.workers"


def create_namespace_module_manager(namespace: str = None) -> NamespaceModuleManager:
    return NamespaceModuleManager(namespace or __NAMESPACE)


def get_all_workers_on_namespace(namespace: str = None) -> List[str]:
    nmm = create_namespace_module_manager(namespace)
    return nmm.get_all_available_module_names()


def load_worker_on_namespace(
    worker_name: str, namespace: str = None
) -> Optional[EntryPoint]:
    worker = None
    nmm = create_namespace_module_manager(namespace)

    try:
        worker = nmm.load_module(worker_name)
    except ValueError:
        print(f"No worker could be loaded by the name {worker_name}.")

    return worker



from functools import partialmethod
import logging
import os

from redis import StrictRedis

logger = logging.getLogger(__name__)


class RedisProxy:
    def __init__(self, backend):
        self.backend = backend

    def proxy(self, method, *args, **kwargs):
        errors = {}

        backend_name = self.backend.__class__.__name__
        try:
            logger.debug(
                "trying to {} {} using {}".format(method, kwargs, backend_name)
            )
            bound_method = getattr(self.backend, method)
            return bound_method(*args, **kwargs)
        except Exception as e:
            if os.getenv("DEBUG"):
                raise
            errors[backend_name] = str(e)

        raise Exception(
            "no backend managed to {} for the provided request: {}".format(
                method, errors
            )
        )

    lpush = partialmethod(proxy, "lpush")
    lrange = partialmethod(proxy, "lrange")
    erase = partialmethod(proxy, "erase")


class RedisDummyBackend:
    def __init__(self, **kwargs):
        self.redis_lists = {}

    def job_exists(self, name):
        return name in self.redis_lists

    def lpush(self, name, elements):
        if not isinstance(elements, list):
            elements = [elements]

        elements = [str(element).encode("utf-8") for element in elements]

        if not name in self.redis_lists:
            self.redis_lists[name] = []

        self.redis_lists[name] = elements + self.redis_lists[name]

    def lrange(self, name, start, end):
        if not self.job_exists(name):
            self.redis_lists[name] = []

        if end == -1:
            return self.redis_lists[name][start:]
        else:
            end = end + 1

        result = self.redis_lists[name][start:end]

        return result

    def erase(self):
        self.redis_lists = {}


def RedisBackend(**kwargs):
    testing = os.getenv("TESTING") or not os.getenv("REDIS_BACKEND")
    if testing:
        flavors = ("dummy",)
    else:
        flavors = (os.getenv("REDIS_BACKEND"),)

    for flavor in flavors:
        backend = {"dummy": RedisDummyBackend, "redis": StrictRedis}[flavor](**kwargs)

    redis_client = RedisProxy(backend)

    return redis_client



import redis

from redis_log_handler.RedisLogHandler import RedisLogHandler
from conftest import LogTestHelper


def test_ping_redis(redis_client: redis.Redis):
    assert redis_client.ping() is True


def test_send_message_to_redis(redis_client: redis.Redis, log_test_helper: LogTestHelper):
    handler = RedisLogHandler('ch:test_channel')
    logger = log_test_helper.generate_logger('logger', 'INFO', handler.channel)
    subscriber = log_test_helper.generate_subscriber_on_channel(redis_client, handler.channel)

    message = subscriber.get_message()
    assert log_test_helper.is_message_data_empty(message) is True

    logger.info('Test logger')
    message = subscriber.get_message()

    assert log_test_helper.is_message_data_empty(message) is False
    assert 'Test logger' in message.get('data').decode('utf-8')

    log_test_helper.close_connection_to_channel(subscriber)


def test_unique_channel_per_handler():
    handler_one = RedisLogHandler('ch:channel_01')
    handler_two = RedisLogHandler('ch:channel_02')
    handler_three = RedisLogHandler('ch:channel_03')

    assert handler_one.channel != handler_two.channel != handler_three.channel


def test_unique_channel_publishing(redis_client: redis.Redis, log_test_helper: LogTestHelper):
    channel_one = 'ch:channel_01'
    channel_two = 'ch:channel_02'
    channel_three = 'ch:channel_03'

    subscriber_one = log_test_helper.generate_subscriber_on_channel(redis_client, channel_one)
    subscriber_two = log_test_helper.generate_subscriber_on_channel(redis_client, channel_two)
    subscriber_three = log_test_helper.generate_subscriber_on_channel(redis_client, channel_three)

    logger_one = log_test_helper.generate_logger('first_logger', 'INFO', channel_one)
    logger_two = log_test_helper.generate_logger('second_logger', 'INFO', channel_two)

    for msg in (subscriber_one.get_message(), subscriber_two.get_message(), subscriber_three.get_message()):
        assert log_test_helper.is_message_data_empty(msg) is True

    # Publish message to first channel
    logger_one.info('Test message on channel 1.')
    for msg in (subscriber_two.get_message(), subscriber_three.get_message()):
        assert log_test_helper.is_message_data_empty(msg) is True

    message_one = subscriber_one.get_message()
    assert log_test_helper.is_message_data_empty(message_one) is False
    assert isinstance(message_one.get('data').decode('utf-8'), str)
    assert 'Test message on channel 1.' in message_one.get('data').decode('utf-8')

    # Publish message to 2nd channel
    logger_two.info('Test message on channel 2.')
    for msg in (subscriber_one.get_message(), subscriber_three.get_message()):
        assert log_test_helper.is_message_data_empty(msg) is True

    message_two = subscriber_two.get_message()
    assert log_test_helper.is_message_data_empty(message_two) is False
    assert isinstance(message_two.get('data').decode('utf-8'), str)
    assert 'Test message on channel 2.' in message_two.get('data').decode('utf-8')

    log_test_helper.close_connection_to_channel(subscriber_one, subscriber_two, subscriber_three)




import pytest
from dotenv import load_dotenv

from skidward.web import app


@pytest.fixture
def init_env_vars():
    load_dotenv(verbose=True)


def test_it_creates_app(init_env_vars):
    assert app



from setuptools import setup


setup(
    name="redis_log_handler",
    version="0.1",
    description="A log handler for the Python logging module, emitting all logs to specific Redis channels.",
    packages=["redis_log_handler"],
)


from unittest import mock
import pytest

import skidward.jobs_handler.jobs_handler_process as jh
from skidward import JobStatus


class TestJobsHandlerProcess:
    @pytest.mark.parametrize("id, expected", [(1, True), (2, False)])
    def test_function_job_exists_return_if_job_present_in_redis_list(
        self, id, expected, backend
    ):
        exist = jh.job_exists(id, backend)

        assert exist is expected

    def test_adding_a_job_calls_redis(self, session, backend):
        result = backend.lrange("jobs", 0, 1)
        assert result[0] == "1".encode("utf-8")

        jh.add_jobs_to_redis(backend, [2], session)

        results = backend.lrange("jobs", 0, 1)
        assert results[0] == "2".encode("utf-8")
        assert session.query.call_count == 2
        assert session.query().filter.call_count == 1
        session.commit.assert_called_once()

    @mock.patch("skidward.jobs_handler.jobs_handler_process.db")
    def test_getting_a_job_when_it_is_ready(
        self, mock_db, session, backend, add_job_to_db
    ):
        assert not jh.get_jobs(backend)
        add_job_to_db(2, JobStatus.READY)
        mock_db.session = session

        jh.get_jobs(backend)

        results = backend.lrange("jobs", 0, -1)
        assert len(results) == 2
        assert results[0] == "2".encode("utf-8")
        assert session.query.call_count == 3
        assert session.query().filter.call_count == 2
        session.commit.assert_called_once()

    @mock.patch("skidward.jobs_handler.jobs_handler_process.db")
    def test_getting_no_job_when_not_ready(self, mock_db, session, backend):
        assert not jh.get_jobs(backend)
        session.query().filter.return_value = []
        mock_db.session = session

        jh.get_jobs(backend)

        # Check there is only an already existing job present in the list
        results = backend.lrange("jobs", 0, -1)
        assert len(results) == 1
        assert results[0] == "1".encode("utf-8")
        assert session.query.call_count == 3



from datetime import datetime as dt

from flask_sqlalchemy import SQLAlchemy
from sqlalchemy_utils import JSONType
from flask_security import UserMixin, RoleMixin

from skidward.web import app
from skidward import JobStatus


# Initializing DB
db = SQLAlchemy(app)

# Creating Models
roles_users = db.Table(
    "roles_users",
    db.Column("user_id", db.Integer(), db.ForeignKey("user.id")),
    db.Column("role_id", db.Integer(), db.ForeignKey("role.id")),
)


class Role(db.Model, RoleMixin):
    id = db.Column(db.Integer(), primary_key=True)
    name = db.Column(db.String(80), unique=True, default="end_user")
    description = db.Column(db.String(255))

    def __repr__(self):
        return self.name


class User(db.Model, UserMixin):
    id = db.Column(db.Integer, primary_key=True)
    email = db.Column(db.String(255), unique=True, nullable=False)
    username = db.Column(db.String(255), unique=True, nullable=False)
    password = db.Column(db.String(255), nullable=False)
    active = db.Column(db.Boolean(), default=True)
    confirmed_at = db.Column(db.DateTime(), default=dt.now())
    roles = db.relationship(
        "Role", secondary=roles_users, backref=db.backref("users", lazy="dynamic")
    )

    def __repr__(self):
        return self.username


class Worker(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(80), unique=True)
    description = db.Column(db.String(255))
    tasks = db.relationship("Task", backref="worker")

    def __repr__(self):
        return self.name


class Task(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(80), unique=True)
    worker_id = db.Column(db.Integer, db.ForeignKey("worker.id"), nullable=False)
    context = db.Column(JSONType, nullable=False)
    cron_string = db.Column(db.String)

    def __repr__(self):
        return self.name


class Job(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    state = db.Column(db.Enum(JobStatus), nullable=False, default=JobStatus.READY)
    ran_at = db.Column(db.DateTime(), default=dt.utcnow())
    task_id = db.Column(db.Integer, db.ForeignKey("task.id"), nullable=False)
    task = db.relationship("Task")

    def __repr__(self):
        return "Job %s at %s is %s. " % (self.task_id, self.ran_at, self.state)



import pytest
import os

from skidward.web import app
from skidward.models import db, User, Worker


def create_app():
    app.config["TESTING"] = os.getenv("TESTING")
    app.config["WTF_CSRF_ENABLED"] = os.getenv("WTF_CSRF_ENABLED")
    app.config["SQLALCHEMY_DATABASE_URI"] = "sqlite:///:memory:"
    app.config["SECRET_KEY"] = os.getenv("SECRET_KEY")
    return app


@pytest.fixture()
def test_client():
    flask_app = create_app()
    client = flask_app.test_client()
    ctx = flask_app.app_context()
    ctx.push()
    yield client
    ctx.pop()


@pytest.fixture()
def init_database():
    db.create_all()
    user1 = User(email="test@test.com", username="testuser", password="123")
    db.session.add(user1)
    worker1 = Worker(name="Test_Worker")
    db.session.add(worker1)
    db.session.commit()
    yield db
    db.drop_all()


@pytest.fixture(autouse=True)
def enable_debugging(monkeypatch):
    monkeypatch.setenv("TESTING", "True")



from flask_login import current_user

from skidward.models import User, Worker


def test_it_loads_the_app(test_client):
    response = test_client.get("/login")
    assert response.status_code == 200


def test_it_requires_login_to_access_home_page(test_client):
    response = test_client.get("/", follow_redirects=True)
    assert b"Please log in to access this page" in response.data


def test_it_logs_in_correct_user(test_client, init_database):
    with test_client:
        response = test_client.post(
            "/login",
            data=dict(email="test@test.com", password="123"),
            follow_redirects=True,
        )
        assert current_user.email == "test@test.com"


def test_it_creates_new_user_in_the_database(test_client, init_database):
    user = User.query.filter_by(email="test@test.com").first()
    assert user is not None
    assert user.email == "test@test.com"
    assert user.username == "testuser"


def test_it_redirects_to_home_on_login(test_client, init_database):
    response = test_client.post(
        "/login",
        data=dict(email="test@test.com", password="123"),
        follow_redirects=True,
    )
    assert b"Scheduler For Python Scripts" in response.data


def test_it_logs_out_correctly_and_redirects_to_login(test_client, init_database):
    response = test_client.get("/logout", follow_redirects=True)
    assert response.status_code == 200
    assert b"Please log in" in response.data


def test_it_creates_worker_and_displays_in_list(test_client, init_database):
    response = test_client.post(
        "/admin/worker/new/", data=dict(name="test_worker"), follow_redirects=True
    )
    assert b"Skidward-Admin" in response.data
    assert b"list" in response.data
    worker = Worker.query.filter_by(name="test_worker").first()
    assert worker is not None
    assert worker.name == "test_worker"



import first_worker


def test_worker_message():
    assert 'first' in first_worker.get_worker_message().lower()


def test_print_worker_message(capsys):
    first_worker.print_worker_message()
    std_out_capture = capsys.readouterr()
    assert std_out_capture.out != ''



import os

from unittest import mock
import pytest
import redis

from skidward.backend import RedisBackend, RedisProxy, RedisDummyBackend


class TestRedisBackend:
    def test_get_expected_backend(self):
        backend = RedisBackend()
        assert backend.__class__.__name__ == "RedisProxy"
        assert backend.backend.__class__.__name__ == "RedisDummyBackend"

    @mock.patch("skidward.backend.StrictRedis")
    def test_get_actual_redis_backend(self, mock_redis):
        # TESTING env var is added via the autouse "enable_testing" fixture
        # Need to remove it to check the real case behavior
        os.environ.pop("TESTING")
        backend = RedisBackend()
        assert backend.__class__.__name__ == "RedisProxy"
        assert backend.backend.__class__.__name__ == "MagicMock"

    @mock.patch("skidward.backend.StrictRedis")
    def test_call_actual_redis_commands(self, mock_redis):
        # TESTING env var is added via the autouse "enable_testing" fixture
        # Need to remove it to check the real case behavior
        os.environ.pop("TESTING")
        backend = RedisBackend()
        backend.lpush("jobs", 1)
        backend.lrange("jobs", 0, -1)

        calls = [
            mock.call(),
            mock.call().lpush("jobs", 1),
            mock.call().lrange("jobs", 0, -1),
        ]

        assert mock_redis.mock_calls == calls


class TestRedisProxy:
    @pytest.mark.parametrize("expected_class", (RedisDummyBackend, redis.Redis))
    def test_proxy_returns_expected_class(self, expected_class):
        backend = RedisProxy(expected_class)
        assert backend.backend == expected_class


class TestRedisDummyBackend:
    @pytest.mark.parametrize("list_name, expected", (("job", True), ("nope", False)))
    def test_job_exists_in_redis_list(self, list_name, expected):
        dummy_backend = RedisDummyBackend()
        assert not dummy_backend.job_exists(list_name)
        dummy_backend.redis_lists["job"] = [1]
        assert dummy_backend.job_exists(list_name) is expected

    @pytest.mark.parametrize("jobs_id", (2, [2]))
    def test_lpush_handles_int_and_list(self, jobs_id):
        dummy_backend = RedisDummyBackend()
        dummy_backend.redis_lists["job"] = ["1".encode("utf-8")]

        dummy_backend.lpush("job", jobs_id)

        assert dummy_backend.redis_lists["job"] == [
            "2".encode("utf-8"),
            "1".encode("utf-8"),
        ]

    @pytest.mark.parametrize("list_name", ("job", "add_me"))
    def test_lpush_creates_redis_list_if_not_existing(self, list_name):
        dummy_backend = RedisDummyBackend()
        assert not dummy_backend.redis_lists

        dummy_backend.lpush(list_name, 1)

        assert dummy_backend.redis_lists
        jobs = dummy_backend.redis_lists.keys()
        assert list(jobs)[0] == list_name

    @pytest.mark.parametrize("values", ([1], [1, 2], [1, 2, 3]))
    def test_lrange_on_an_existing_job_returns_values(self, values):
        dummy_backend = RedisDummyBackend()
        assert not dummy_backend.lrange("job", 0, -1)
        dummy_backend.redis_lists["job"] = values

        result = dummy_backend.lrange("job", 0, -1)

        assert values == result

    @pytest.mark.parametrize(
        "start, end, expected",
        ((0, -1, [1, 2, 3, 4, 5]), (0, 3, [1, 2, 3, 4]), (1, 3, [2, 3, 4])),
    )
    def test_lrange_returns_expected_segment(self, start, end, expected):
        dummy_backend = RedisDummyBackend()
        assert not dummy_backend.lrange("job", start, end)
        dummy_backend.redis_lists["job"] = [1, 2, 3, 4, 5]

        result = dummy_backend.lrange("job", start, end)

        assert result == expected

    @pytest.mark.parametrize(
        "start, end, expected",
        (
            (-10, 0, [1]),
            (-10, 10, [1, 2, 3, 4, 5]),
            (0, -10, []),
            (6, 10, []),
            (4, 2, []),
        ),
    )
    def test_lrange_when_start_end_are_out_of_bounds(self, start, end, expected):
        dummy_backend = RedisDummyBackend()
        assert not dummy_backend.lrange("job", start, end)
        dummy_backend.redis_lists["job"] = [1, 2, 3, 4, 5]

        result = dummy_backend.lrange("job", start, end)

        assert result == expected

    def test_erase_deletes_redis_list(self):
        dummy_backend = RedisDummyBackend()
        dummy_backend.redis_lists["job"] = [1, 2, 3, 4, 5]
        assert dummy_backend.redis_lists
        dummy_backend.erase()
        assert not dummy_backend.redis_lists

    def test_exception(self):
        dummy_backend = RedisDummyBackend()
        with pytest.raises(AttributeError) as attr_error:
            dummy_backend.fake_function()
        assert "'RedisDummyBackend' object has no attribute 'fake_function'" == str(
            attr_error.value
        )



import logging

import redis


class RedisLogHandler(logging.StreamHandler):
    def __init__(self, channel: str, host: str = None, port: int = None, password: str = None):
        super().__init__()

        self.channel = channel
        self.host = host or 'localhost'
        self.port = port or 6379
        self.password = password or None

    def emit(self, message: logging.LogRecord):
        redis_client = redis.Redis(host=self.host, port=self.port, password=self.password)
        redis_client.publish(self.channel, str(message))
